# -*- coding: utf-8 -*-
"""
Created on Fri May 20 12:59:40 2016

@author: gjtim
"""

# Imports
from matplotlib.path import Path
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import math
import matplotlib.path as mpath
import matplotlib.image as mpimg
import matplotlib.animation as manimation
import matplotlib.font_manager as fm

#FFMpegWriter = manimation.writers['ffmpeg']
#metadata = dict(title='Movie Test', artist='Matplotlib',
#                comment='Movie support!')
#writer = FFMpegWriter(fps=10, metadata=metadata)

""" Physics Constants """
pi = 3.14159
sy = 3.154e+7       # Seconds in a year
h  = 6.626e-34      # Planck's Constant   JouleSeconds
hbar = 1.0545e-34   # JouleSeconds
k  = 1.3805e-23     # Joules per kelvin  (Boldtzman constant)

amond = 1.2e-10     # Mond Acceleration threshold m/s2
G  = 6.673e-11      # m3s-2kg-1
c  = 2.9979e8       # metre/sec  
qel = 1.6022e-19    # Elementary Charge (coulomb)
e0 = 8.8542e-12     # Permittivity of free space  F·m−1 (farads per metre)
e = np.sqrt(qel**2/(4 * pi * e0))
phi0 = 2.06783461e-15 # Magnetic flux quantum (volt-second),can be changed to Hz/Volt
alpha = qel**2/(4*pi*e0*hbar*c)    # Fine structure constant
me = 9.11e-31       # Electron mass (kg)
mp = 1.67e-27       # Proton mass (kg)
opsj = 2/h          # Operations per second per joule = 3.018e33
opskg = opsj*c*c    # Operations per second per kilogram = 5.4258e+50 MARGOLUS-LEVITIN

rb = 5.28e-11       # Bohr radius H atom (metre)
rp = 0.8775e-15       # Proton radius (metre) # From electron scattering
cre= qel**2/(4*pi*e0*me*c**2) # Electron Radius Classical. The classical radii of protons and electrons are the distances where the electromagnetic self energy of the proton and electron is equal to their respective masses
#crp= alpha*hbar/(mp*c) # Proton Radius Classical. (But the proton not fundamental so WRONG!)
Comptonwp = h/(mp*c)  # Compton wavelength of proton
Comptonwe = h/(me*c)  # Compton wavelength of electron
Comptonre = hbar/(me*c)  # Compton radius of electron

opskg = 5.4258e50   # Ops per second per kilogram

lpl= 1.616e-35      # Planck length (metre)
tpl= 5.391e-44      # Planck time 
mpl= 2.177e-8       # Planck mass (kg)
engpl = 1.96e9      # Planck Energy (Joules)
tmppl = 1.41e32     # Planck Temperature (K)
mpls= (mpl/1000)**(1/3)



aou = 13.799e9 * sy # Universe age (seconds) = 13.8 billion years
ru  = aou*c         # Universe radius (non-comoving) of universe = 1.3e26 metres
duc = 8.8e26        # Universe observable diameter comoving (metres) = 93 billion years
ruc = 4.4e26        # Universe Observable radius comoving (metres)  = 46.5 billion ly
mu = 1.8e53         # Universe mass (kg) from C^3=GM/t
cc = 3.1e-122       # Cosmological Constant (per area unit)
cu = math.sqrt(ru/lpl)*lpl # Centre of the universe (m) = sqrt(ru) * sqrt(lpl)
cbrw = 1.06e-3      # Cosmic background wavelength (m)
visw = 4.9e-7       # Visible Spectrum Wavelength (m)
ryd = 2.1799e-18    # Rydberg energy (Joules)
kb  = 1.38065e-23   # Boltzman constant
mgal = 6e42         # Mass of Milky Way galaxy including halo
 
engdenspl = mpl/(lpl**3)    # Energy density planck (kg/m3)  Should div by 4/3 pi r3 for volume
engdensu = mu/(ru**3)       # Energy density lf universe (kg/m3)
engdensmin = 7e-70/(ru**3)  # Energy density min (particle spanning universe) (kg/m3) majid
engdensp = mp/(rp**3)       # Energy density of proton
engdensa = (mp+me)/(rb**3)  # Energy density of Hydrogen atom = 11351 kg/m3, 11 times water!
                            # aprox = lead = 11.34 x water
                            
rangesf = 2e-15     # Range of Strong force
rangewf = 2e-18     # Range of Weak force

CBRwavelength = 1.06e-3     #(m)
CBRtemp = 2.726     # Kelvin
VisibleSpectrumWavelength = 4.9e-7  #(m)
UVWavelength = 2e-8
XRayWavelength = 1e-9
GammaRayWavelength = 1e-12
InfraRedWavelength = 1e-5
GravityMeasuredLength = 1e-4

StarSize = 7e8      # Star size in metres
GalaxySize = 1e21   # Galaxy size in metres
MONDacc = 1.2e-10   # persec persec (1.28x10-8cm/s/s)
MONDl = c*c/MONDacc # Mond length 7.489503675e+26 - off the chart
MONDm = c**4/(G*MONDacc)    # Mond mass 1.0087e+54 - heavier than the universe
MONDt = c/MONDacc   # Mond time 2.49825e+18s - longer than hubble time
engdensgal = mgal/(GalaxySize**3) # Energy density of galaxy

LHCscale = 1e-17    # metres (13 Tev)
INFstartscale = 1e-28   # Inflation start time
INFendscale = 1e-24     # Inflation end time
BloodCell = 1e-5    # Human blood cell size

GUT_Epoch = 1e-36
Inflation_Epoch = 1e-33
Electro_Weak_Epoch = 1e-32


# Black hole temperature = hbar * c**3 / 8 pi G M kb
BHTpl = hbar*c**3/(8*pi*G*mpl*kb)
BHTu  = hbar*c**3/(8*pi*G*mu*kb)
UnruhAccU = 2*pi*c*kb*BHTu/hbar     # Unruh acceleration from universe as BH Temp
MONDtmp = MONDacc*hbar/(2*pi*c*kb)
BHrpl = 2*G*mpl/(c*c)   # Planck Black hole gravitational radius from its mass

# Angular momentum
AM_Sun = 1.8838e41          # Angular momentum of sun
Rad_Sun = 6.9634e8          # Radius of the sun
AM_SolarSystem = 3.3212e45  # Angular momentum of solar system  kgm2/sec
Rad_SolarSystem = 4.545e12  # Radius of solar system to Neptune 
AM_MilkyWay = 1e67          # Angular momentum of our galaxy
Rad_MilkyWay = 5e20         # Radius of our galaxy

# Electromagnetic view
HCap  = 4*pi*2*e0*rb  # Capacitance of Hydrogen atom

# Computation
OrthogonalTime = pi*hbar/(2*me*c*c) # Min for orthogonal state, electron average mass-energy
 
# Set canvas size in inches
pltwid = 14       # For 1920 x 1080 HD video
pltht = 14
margin = 0.5
lm = 1
rm = pltwid - lm
bm = margin
tm = pltht - margin 


pows10 = 61             # Scale Range in powers of 10
y1 = (tm-bm)/pows10     # Y interval for 1 power of 10
rad = (pltht-2*margin)/2


# ************** Initialize the canvas *****************************
# Create a new figure of size pltwid x pltht inches, using 100 dots per inch
fig = plt.figure(figsize=(pltwid,pltht), dpi=100,facecolor='white')    # doesn't work with animation writer saving

    
plt.xlim(0,pltwid)      # Set x limits
plt.xticks(np.linspace(0,10,0,endpoint=True))   # Set x ticks
# plt.xticks(np.linspace(start, finish,Number of ticks,endpoint+True))
plt.ylim(0,pltht)       # Set y limits
plt.yticks(np.linspace(0,10,0,endpoint=True))   # Set y ticks
plt.axis('off')         # axis off and tight make it all bigger
plt.tight_layout()


 
# ***************************** Functions *********************************
    
def Rotate (xc, yc, xr, yr, ang):
    return xc + (xr-xc)*math.cos(ang) - (yr-yc)*math.sin(ang), yc + (xr-xc)*math.sin(ang) + (yr-yc)*math.cos(ang)

def truncate(f, n):
    '''Truncates/pads a float f to n decimal places without rounding'''
    s = '%.12f' % f
    i, p, d = s.partition('.')
    return '.'.join([i, (d+'0'*n)[:n]])

def draw_sausage(xc1,xc2,yc1,yc2,thick,col):
    
    dx = xc2 - xc1
    dy = yc2 - yc1
    if dx == 0:
        if dy > 0:
            ang = pi/2
        else:
            ang = 3*pi/2
    elif dy == 0:
        ang = 0
    else:
        if dx > 0 and dy > 0:
            ang = math.atan(dy/dx)
        elif dx < 0 and dy > 0:
            ang = pi - math.atan(dy/(-dx))
        elif dx > 0 and dy < 0:
            ang = 2*pi - math.atan((-dy)/dx)
        elif dx < 0 and dy < 0:
            ang = pi + math.atan((-dy)/(-dx))
    dx = 0.5*thick*math.sin(ang)
    dy = 0.5*thick*math.cos(ang)

# Draw a path with straight lines and curves
    verts = [
    (xc1-dx, yc1+dy),             # Start Position
    (xc2-dx, yc2+dy),             # End of 1st straight line
    (xc2-dx+1.33*dy, yc2+dy+1.33*dx),    
    (xc2+dx+1.33*dy, yc2-dy+1.33*dx),    
    (xc2+dx, yc2-dy),             # End of curve
    (xc1+dx, yc1-dy),             # End of 2nd straight line
    (xc1+dx-1.33*dy, yc1-dy-1.33*dx),    
    (xc1-dx-1.33*dy, yc1+dy-1.33*dx),        
    (xc1-dx, yc1+dy),             # Finish back at start    
    #(x+0.9*idlerstepx+dx/2, y-0.9*idlerstepy+dy/2),       # Control point curve2
    #(x, y),                                       # End of curve2 - FINISH
    ]
    codes = [Path.MOVETO,
         Path.LINETO,
         Path.CURVE4,
         Path.CURVE4,
         Path.CURVE4,
         Path.LINETO,         
         Path.CURVE4,
         Path.CURVE4,
         Path.CURVE4,
    
     ]
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(path, edgecolor=col, facecolor=col)
    plt.gca().add_patch(patch) 


def draw_rectangle(xc1,xc2,yc1,yc2,thick,col):
    # From horizontal centre of each end !!
    
    dx = xc2 - xc1
    dy = yc2 - yc1
    if dx == 0:
        if dy > 0:
            ang = pi/2
        else:
            ang = 3*pi/2
    else:
        if dx > 0 and dy > 0:
            ang = math.atan(dy/dx)
        elif dx < 0 and dy > 0:
            ang = pi - math.atan(dy/(-dx))
        elif dx > 0 and dy < 0:
            ang = 2*pi - math.atan((-dy)/dx)
        elif dx < 0 and dy < 0:
            ang = pi + math.atan((-dy)/(-dx))
    dx = 0.5*thick*math.sin(ang)
    dy = 0.5*thick*math.cos(ang)

# Draw a path with straight lines and curves
    verts = [
    (xc1-dx, yc1+dy),             # Start Position
    (xc2-dx, yc2+dy),             # End of 1st straight line
    (xc2+dx, yc2-dy),             # End of 2nd straight line
    (xc1+dx, yc1-dy),             # End of 3rd straight line
    (xc1-dx, yc1+dy),             # Finish back at start    
    ]
    codes = [Path.MOVETO,
         Path.LINETO,
         Path.LINETO,         
         Path.LINETO,
         Path.LINETO,    
     ]
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(path, edgecolor=col, facecolor=col,)
    plt.gca().add_patch(patch) 


def Draw_Man(xc, yc, ang, scale):
    # Set man height as 1
    
    headradius = 0.15 * scale
    #bodywidth = 0.37 * scale
    armoff = 0.09 * scale
    armoff2 = 0.38 * scale
    armwidth = 0.12 * scale
    legwidth = armwidth

    # Fixed points of man
    headcx = xc
    headcy = yc + 0.5*scale
    armcxr = xc+armoff
    armcxl = xc-armoff
    armcx2r = xc+armoff2
    armcx2l = xc-armoff2
    armcyr = yc + 0.265*scale
    armcyl = armcyr
    armcy2r = yc + 0.505*scale
    armcy2l = armcy2r
    #legcxr = armcxr
    #legcyr = armcyr
    legcx2r = armcxr
    legcx2l = armcxl
    legcy2r = yc-0.59*scale
    legcy2l = legcy2r
    bodytopx = xc
    bodytopy = yc+0.3175*scale
    bodybotx = xc
    bodyboty = yc-0.17*scale
 
   
    if ang > 0:
        # Rotate the points about the centre at xc, yc
        headcx, headcy = Rotate (xc, yc, headcx, headcy, ang)
        armcxr, armcyr = Rotate (xc, yc, armcxr, armcyr, ang)
        armcx2r, armcy2r = Rotate (xc,yc, armcx2r, armcy2r, ang)
        armcxl, armcyl = Rotate (xc, yc, armcxl, armcyl, ang)
        armcx2l, armcy2l = Rotate (xc,yc, armcx2l, armcy2l, ang)
        legcx2l, legcy2l = Rotate (xc, yc, legcx2l, legcy2l, ang)
        legcx2r, legcy2r = Rotate (xc, yc, legcx2r, legcy2r, ang)
        bodytopx, bodytopy = Rotate (xc, yc, bodytopx, bodytopy, ang)
        bodybotx, bodyboty = Rotate (xc, yc, bodybotx, bodyboty, ang)
        
    # Draw Head
    circle = mpatches.Circle((headcx, headcy), headradius, facecolor='black', linewidth=0)
    plt.gca().add_patch(circle)

    # Draw Arms
    draw_sausage(armcxr, armcx2r, armcyr, armcy2r, armwidth, 'black')
    draw_sausage(armcxl, armcx2l, armcyl, armcy2l, armwidth,'black')    
    
    # Draw Legs
    draw_sausage(armcxr, legcx2r, armcyr, legcy2r, legwidth,'black')
    draw_sausage(armcxl, legcx2l, armcyl, legcy2l, legwidth,'black')    
    
    # Draw Body
    draw_rectangle(bodytopx, bodybotx, bodytopy, bodyboty, headradius,'black')    

def Draw_Star(xc,yc,scale,spikes):
    # It is hideously complicated to create a 'cleaned' list of verts and codes
    # so I generate it with a program and insert the numbers
    
    oradius = scale
    iradius = 0.75 * scale
    n = 2*spikes+1
    
    #Draw a path with straight lines and curves
    #Path = mpath.Path    
    codes = np.ones(n, dtype=mpath.Path.code_type) * mpath.Path.LINETO
    codes[0] = mpath.Path.MOVETO    
    verts = np.zeros((n,2))

    
    for z in range (spikes+1):
        # Add outer point to path
        xe, ye = Rotate (xc, yc, xc, yc+iradius, z*2*pi/spikes)
        verts[2*z] = [truncate(xe,3), truncate(ye,3)]        
           
        if z < spikes:
            # Add inner point to path
            xe, ye = Rotate (xc, yc, xc, yc+oradius, (z+0.5)*2*pi/spikes)
            verts[2*z+1] = [truncate(xe,3), truncate(ye,3)]             

    path = mpath.Path(verts, codes)    
    patch = mpatches.PathPatch(path, edgecolor='orange', facecolor='yellow', linewidth=1)
    plt.gca().add_patch(patch) 
 
def Make_Spiral(xc,yc,a,b,color):
    # Draw Arm - Archimedean spiral radius proportional to angle + const: r = a + b*theta
    # so constants a and c control the spiral

    # Calculate points along spiral in centre of arm
    armpts = 17
    axys = np.zeros((armpts+1,2))     # xys of centre points along arm
    cxys = np.zeros((armpts+1,2))   # xys of control points between centres along arm
    slope = 0
    for i in range(armpts+1):
        # Calculate x,y coordinates for centre points along arm
        ang = i*2*pi/16     # arbitrary breaking of circle into 16 sectors
        r = a + b*ang
        axys[i] = [xc + r*math.sin(ang) , yc + r*math.cos(ang)]    
#        circle = mpatches.Circle((axys[i]), 0.2, facecolor='yellow', linewidth=0)
#        plt.gca().add_patch(circle)                
#        plt.text(axys[i,0], axys[i,1], i, verticalalignment='center', fontsize=12)
        
        # Calculate slope at this point:
        # dy/dx = (btanθ+(a+bθ))/(b−(a+bθ)tanθ)
        oldslope = slope
        slope = 1 / ((b*math.tan(ang) + a + b*ang)/(b - (a + b*ang)*math.tan(ang)))
        # Draw the tangent line
        #plt.plot([axys[i,0]-1, axys[i,0]+ 1], [axys[i,1]-slope, axys[i,1]+slope], 'red', linestyle = '-', lw=1) 

        if i > 0:
            # Calculate intercept of this tangent and last to be bezier control point
            c1 = axys[i,1] - slope*axys[i,0]        # c = y - m*x
            c2 = axys[i-1,1] - oldslope*axys[i-1,0]
            cxys[i,0] = (c2 - c1)/(slope - oldslope)
            cxys[i,1] = slope*cxys[i,0] + c1
            #circle = mpatches.Circle((cxys[i]), 0.1, facecolor='green', linewidth=0)
            #plt.gca().add_patch(circle)
            
    #Draw a path with straight lines and curves
    #Path = mpath.Path    
    codes = np.ones(2*armpts+1, dtype=mpath.Path.code_type) * mpath.Path.CURVE3
    codes[0] = mpath.Path.MOVETO    
    verts = np.zeros((2*armpts+1,2))

    # Create the path
    for z in range (armpts+1):
        # Add outer point to path
        #xe, ye = Rotate (xc, yc, xc, yc+iradius, z*2*pi/spikes)
        verts[2*z] = [axys[z,0],axys[z,1]]        
           
        if z > 0:
            # Add bezier control point to path
            #xe, ye = Rotate (xc, yc, xc, yc+oradius, (z+0.5)*2*pi/spikes)
            verts[2*z-1] = [cxys[z,0],cxys[z,1]]             

        #    path = mpath.Path(verts, codes)    
        #    patch = mpatches.PathPatch(path, edgecolor=color, facecolor='none', linewidth=1)
        #    plt.gca().add_patch(patch) 
    
    return verts, codes
    #print ('verts=',verts,' Codes =',codes)        
    
def Make_Spiral_Arm(xc, yc, scale):
    # Combine the two spiral paths to make tapering spiral wedge
    verts1, codes = Make_Spiral(xc, yc, scale, scale, 'red')
    verts2, codes = Make_Spiral(xc, yc, 2*scale, 0.85*scale, 'blue')


    n = len(verts1)
    n1 = 2*n
    verts = np.zeros((n1,2))
    codes = np.ones(n1, dtype=mpath.Path.code_type) * mpath.Path.CURVE3
    
    for z in range(n):
        # Put first path in new array forwards
        verts[z] = [verts1[z,0], verts1[z,1]]
    
    #print (z)
    codes[0] = mpath.Path.MOVETO
    codes[n] = mpath.Path.LINETO      # To join the tips of the spirals
    for z in range(n, n1):
        # Put second path in new array backwards
        verts[z] = [verts2[n1-z-1,0], verts2[n1-z-1,1]]

    return verts, codes

def Draw_Arm(xc, yc, scale, ang, color):

    verts, codes = Make_Spiral_Arm(xc, yc, scale)    
    # Rotate verts
    for z1 in range(len(verts)):
        verts[z1] = Rotate (xc, yc, verts[z1,0], verts[z1,1], ang) 
    
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(path, edgecolor='none', facecolor=color, linewidth=0)
    plt.gca().add_patch(patch)
        
def Draw_Galaxy(xc, yc, scale, color):
    # Print circular hub of galaxy
    circle = mpatches.Circle((xc, yc), 1.99*scale, facecolor=color, linewidth=0)
    plt.gca().add_patch(circle)

    # Draw for each of 4 copies of galaxy arm   
    # Must draw each arm in separate function call to stop Matplotlib using same path again.
    #verts0 = np.copy(verts)
    ang = 0
    for _ in range (4):
        Draw_Arm(xc ,yc, scale, -ang, color)
        ang += pi/2
    

        
""" *******************************************************************************"""    
""" ************************************ Main Program *****************************"""
""" *******************************************************************************""" 
#Ucolor = '#99ff99'  # Pale green
Ucolor = '#aaffaa'  # Pale lime green
Acolor = '#ff6622'  #'#ff3333'  Atom-Nucleus band
Ncolor = '#d9d9d9'  # Grey
Icolor = '#ff8080'  # Inflation epoch
  
# Draw Universe Circle
# This is fitted to the margins
radius = (pltht-2*margin)/2
circle = mpatches.Circle((pltwid/2, pltht/2), radius, edgecolor='black',
                          facecolor='none',linewidth=1)
plt.gca().add_patch(circle)
## Draw blank rectangle over line where label is to go
#width = 1
#height = 1
#x = pltwid/2 - width/2
#y = pltht/2 + radius - height/2
#blanker = mpatches.Rectangle((x, y), width, height, color='white')
#plt.gca().add_patch(blanker)
# Now draw interior fill
circle = mpatches.Circle((pltwid/2, pltht/2), radius, edgecolor='none',
                          facecolor=Ucolor,linewidth=0)
plt.gca().add_patch(circle)
# Label Universe
y = (tm-bm)/2 + bm + radius
dy = 0.26
dx = 0.6
draw_sausage(pltwid/2-dx, pltwid/2+dx, y+0.02, y+0.02, dy, 'white')
plt.text(pltwid/2, y,'Universe', verticalalignment='center',
         horizontalalignment='center', backgroundcolor='none',fontsize=20)

# Draw radial line
#y = (tm-bm)/2 + bm
#x = (rm-lm)/2 + lm
#plt.plot([x, x], [y, y+radius], 'black', linestyle = '-', lw=3) 
#plt.plot([x, x], [y, y+radius + margin/2], 'black', linestyle = '-', lw=2) 
#plt.plot([x, x+.15], [y+radius + margin/2, y+radius + margin/2 -.2], 'black', linestyle = '-', lw=2) 
#plt.plot([x, x-.15], [y+radius + margin/2, y+radius + margin/2 -.2], 'black', linestyle = '-', lw=2) 


# Draw Galaxy circle
radg = math.log10(GalaxySize/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radg, edgecolor='black',
                          facecolor='none',linewidth=1)
plt.gca().add_patch(circle)
# Label Galaxy
y = (tm-bm)/2 + bm +radg
plt.text(pltwid/2, y,'Galaxy', verticalalignment='center',
         horizontalalignment='center', backgroundcolor=Ucolor, fontsize=14)
mc = 14 # Number of galaxies to draw
for z in range(1,mc):
    ang = 2*pi*z/mc
    mx = pltwid/2
    my = pltht/2 + radg
    mx, my = Rotate (pltwid/2, pltht/2, mx, my, ang)
    Draw_Galaxy(mx, my, 0.03, '#ff0099')

# Draw Star circle
radst = math.log10(StarSize/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radst, edgecolor='black',
                          facecolor='none',linewidth=1)
plt.gca().add_patch(circle)
# Label Star
y = (tm-bm)/2 + bm +radst
plt.text(pltwid/2, y,'Star', verticalalignment='center',
         horizontalalignment='center', backgroundcolor=Ucolor,fontsize=14)
#mc = 13 # Number of stars to draw
for z in range(1,mc):
    ang = 2*pi*z/mc
    mx = pltwid/2
    my = pltht/2 + radst
    mx, my = Rotate (pltwid/2, pltht/2, mx, my, ang)
    Draw_Star(mx, my, 0.2, 23)
    
# Draw Man circle
radd = math.log10(1/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radd, edgecolor='black',
                          facecolor='none',linewidth=1)
plt.gca().add_patch(circle)
# Label Man
y = (tm-bm)/2 + bm +radd
dy = 0.26
dx = 0.25
draw_sausage(pltwid/2-dx, pltwid/2+dx, y+0.02, y+0.02, dy, 'white')
plt.text(pltwid/2, y,'Man', verticalalignment='center',
         horizontalalignment='center', backgroundcolor='none', fontsize=20)
#mc = 13 # Number of men to draw
for z in range(1,mc):
    ang = 2*pi*z/mc
    mx = pltwid/2
    my = pltht/2 + radd + 0.02
    mx, my = Rotate (pltwid/2, pltht/2, mx, my, ang)
    Draw_Man(mx, my, ang, 0.35)     # Last term is scale
    

# Draw CBR circle
radd = math.log10(CBRwavelength/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radd, edgecolor='red',
                          facecolor='none',linewidth=1, linestyle='dotted')
plt.gca().add_patch(circle)
# Label CBR
y = (tm-bm)/2 + bm +radd -0.02
plt.text(pltwid/2+0.01, y,'C.B.R.', verticalalignment='center',
         horizontalalignment='center', backgroundcolor=Ucolor, fontsize=11)
    
# Draw Dust circle
radd = math.log10(10**-5/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radd, edgecolor='black',
                          facecolor='none',linewidth=2, linestyle='dotted')
plt.gca().add_patch(circle)
# Label Dust
y = (tm-bm)/2 + bm +radd
plt.text(pltwid/2, y,'Dust', verticalalignment='center',
         horizontalalignment='center', backgroundcolor=Ucolor,fontsize=11)
#t=plt.text(pltwid/2, y,'Dust', verticalalignment='center',
#         horizontalalignment='center', backgroundcolor=Ucolor,fontsize=11)
#t.set_bbox(dict(facecolor='none', alpha=0.5, edgecolor='none'))
    
# Draw Visible light circle as rainbow colours
diamvl = 2*math.log10(visw/lpl) / math.log10(ruc/lpl) *  radius
theta = 0
dtheta = 360/(12*7)
for z in range(12):
    for clr in range(7):
        if clr == 0:
            col = '#9400D3'     # Violet
        elif clr == 1:
            col = '#4B0082'     # Indigo
        elif clr == 2:
            col = '#0000FF'     # Blue
        elif clr == 3:
            col = '#00FF00'     # Green
        elif clr == 4:
            col = '#FFFF00'     # Yellow       
        elif clr == 5:
            col = '#FF7F00'     # Orange
        elif clr == 6:
            col = '#FF0000'     # Red
            
        vl = mpatches.Arc((pltwid/2, pltht/2), diamvl, diamvl, theta1=theta, theta2=theta+dtheta, edgecolor=col, lw=1.5)
        plt.gca().add_patch(vl)
        theta += dtheta
        #print (col)
        
# Label Visible Light
y = (tm-bm)/2 + bm + diamvl/2 - 0.05
plt.text(pltwid/2, y,'Light', verticalalignment='center',
         horizontalalignment='center', backgroundcolor=Ucolor,fontsize=11)

# Draw Planck Circle
circle = mpatches.Circle((pltwid/2, pltht/2), .02, edgecolor='red',
                          facecolor='red',linewidth=2, zorder=10)
plt.gca().add_patch(circle)
# Label Planck
y = (tm+bm)/2
plt.text(pltwid/2, y+.08,'Planck', horizontalalignment='center', backgroundcolor='none',fontsize=14, zorder=10)

# Draw Atom circle
rada = math.log10(rb/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), rada,
                          facecolor=Acolor,linewidth=0)
plt.gca().add_patch(circle)
# Label Atom
y = (tm-bm)/2 + bm + rada - 0.03
dy = 0.26
dx = 0.32
draw_sausage(pltwid/2-dx, pltwid/2+dx, y+0.02, y+0.02, dy, 'white')
plt.text(pltwid/2, y,'Atom', verticalalignment='center',
         horizontalalignment='center', backgroundcolor='none',fontsize=20)

# Draw Nucleus circle
radn = math.log10(rp/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radn,
                          facecolor=Ncolor,linewidth=0)
plt.gca().add_patch(circle)
# Label Nucleus
y = (tm-bm)/2 + bm + radn
plt.text(pltwid/2, y,'Nucleus', verticalalignment='center',
         horizontalalignment='center', backgroundcolor='none',fontsize=12)

# Draw Question Mark in middle
#path = 'c:\\windows\\fonts\\liberationsans-bolditalic.ttf'
#prop = fm.FontProperties(fname=path)
x = pltwid/2 + 0.07 #0.04
y = pltht/2 - 1.37    #0.8
plt.text(x, y, '?', horizontalalignment='center', color='white', fontsize=280) # was 180

# Draw LHC circle
radLHC = math.log10(LHCscale/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radLHC, edgecolor='black',
                          facecolor='none',linewidth=1, linestyle='dashdot')
plt.gca().add_patch(circle)
# Label LHC
y = (tm-bm)/2 + bm +radLHC - 0.05
plt.text(pltwid/2, y,'L.H.C.', verticalalignment='center',
         horizontalalignment='center', backgroundcolor=Ncolor,fontsize=12)

# Draw Zitter radius circle
radzit = math.log10(Comptonre/(2*lpl)) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radzit, edgecolor='white',
                          facecolor='none',linewidth=1)
plt.gca().add_patch(circle)
# Label Compton radius ZITTER
y = (tm-bm)/2 + bm + radzit - 0.01
plt.text(pltwid/2, y,'Zitter', verticalalignment='center',
         horizontalalignment='center', color='black', backgroundcolor='None',fontsize=12)

# Draw Inflation end circle
radinf = math.log10(INFendscale/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radinf, edgecolor='red',
                          facecolor=Icolor,linewidth=0, linestyle='dashdot', alpha=0.2)
plt.gca().add_patch(circle)
# Draw Inflation start circle
radinf = math.log10(INFstartscale/lpl) / math.log10(ruc/lpl) *  radius
circle = mpatches.Circle((pltwid/2, pltht/2), radinf, edgecolor='red',
                          facecolor=Ncolor,linewidth=0, linestyle='dashdot')
plt.gca().add_patch(circle)
# Label Inflation epoch
y = (tm-bm)/2 + bm +radinf + 0.2
plt.text(pltwid/2, y,'Inflation', verticalalignment='center',
         horizontalalignment='center', backgroundcolor='none',fontsize=12)

# Draw scale
y = (tm-bm)/2 + bm + 0.005
x = (lm + rm)/2
x1 = x + radius + margin
# Draw main arrow line to Radius
plt.plot([x, x+radius], [y, y], 'black', linestyle = '-', lw=1.5, zorder=10) 
# Draw sticking out arrow tip
#arrow_colour = 'black'
arrow_colour = 'white'
plt.plot([x+radius, x1], [y, y], color=arrow_colour, linestyle = '-', lw=1.5, zorder=10) 
plt.plot([x1, x1-.2], [y, y+.13], color=arrow_colour, linestyle = '-', lw=1.5) 
plt.plot([x1, x1-.2], [y, y-.13], color=arrow_colour, linestyle = '-', lw=1.5) 
# Scale label
plt.text(x1-.25, y-.25,'Scale', color=arrow_colour, verticalalignment='center',
         horizontalalignment='center', fontsize=12)
# Ticks
le = 0
yt = y-.25
tl = 0.03
scalefactor = 60/math.log10(ru/lpl)
for tics in range(0,13):
    # Draw tick
    xt = x + tics*radius*scalefactor/12 - 0.01
    plt.plot([xt, xt], [y-tl, y+tl], 'black', linestyle = '-', lw=1)
    # Label scale
    if tics == 0:
        plt.text(xt, yt, '1', horizontalalignment='center', fontsize=13)
    elif tics >= 6 and tics < 12:
        sz =  '$10^{' + str(le) + '}$'   
        plt.text(xt, yt, sz, horizontalalignment='center', backgroundcolor=Ucolor, fontsize=13)
    else:
        sz =  '$10^{' + str(le) + '}$'   
        xt =xt -0.06
        plt.text(xt, yt, sz, horizontalalignment='center', fontsize=13)
    le += 5

# Draw -ve charges on atomic surface
marks = 32
for z in range (1,marks):
    x = pltwid/2 + rada*math.sin((z*2*pi)/marks)
    y = pltht/2 + rada*math.cos((z*2*pi)/marks)
    plt.text(x-.024,y+.007,'-',fontsize=34, color='blue', rotation=str(-z*360/marks),
    verticalalignment='center', horizontalalignment='center')

# Draw +ve charges on nucleus surface
marks = 30
radna = radn - 0.015
for z in range (2,marks-1):
    x = pltwid/2 + radna*math.sin((z*2*pi)/marks)
    y = pltht/2 + radna*math.cos((z*2*pi)/marks)
    if z != 8:
        plt.text(x-.026,y+.011,'+',fontsize=20, color='red', rotation=str(-z*360/marks),
    verticalalignment='center', horizontalalignment='center')


plt.savefig('Atom-Universe-Man.png', transparent=True,dpi=300)
    
# *********************** Main Animation Loop **************************
#
#with writer.saving(fig, "Maxwell Cells Animation.mp4", 100):    # The 100 is dpi
#
#    for i in range(251):        # To leave battery current idlers filling channel
#        plt.clf()
#        setplot()
#
#        # Update code here
#          
#        writer.grab_frame()
#           

   

# ********************** End of Program ************************** 

## Save figure using 100 dots per inch
##Saves file to C:\Users\gjtim\Documents\Python\notebooks
#plt.savefig('Maxwell Animation.png',dpi=100)
#
## Show result on screen
#plt.show()
                 
#myDirname = os.path.normpath("C:/Users/gjtim/Documents/Python/"

#plt.clf()
#circle=plt.Circle((5,3), radius=2, fc='y')
#plt.gca().add_patch(circle)

# ******************* Sample code pieces *************************
"""
# Display Text
#   plt.text(x, y,'text', horizontalalignment='center',fontsize=24)

# Draw Line
    plt.plot([xstart, xfinish], [ystart, yfinish], 'black', lw=3)

# Draw Circle (white for blanking)
    circle = mpatches.Circle((x, y), radius, edgecolor='black', linewidth=3, facecolor='white')
    plt.gca().add_patch(circle)

# Draw Rectangle (white for blanking)
    blanker = mpatches.Rectangle((x, y), width, height, color='white')
    plt.gca().add_patch(blanker)

# Draw Ellipse
    ellipse = mpatches.Ellipse((x,y), width, height, edgecolor='black', linewidth=3, fill=False)
    plt.gca().add_patch(ellipse) 

# Draw Hexagon
    hexagon=mpatches.RegularPolygon((x,y), No of vertices, radius, edgecolor='black', facecolor='white')
    plt.gca().add_patch(hexagon)

# Draw a path with straight lines and curves
    verts = [
    (x, y),                                       # Start Position
    (x-3*idlerstepx, y+3*idlerstepy),             # End of 1st straight line
    (x-3.9*idlerstepx+dx/2, y+3.9*idlerstepy+dy/2),   # Control point curve1
    (x-3*idlerstepx+dx, y+3*idlerstepy+dy),       # End of curve1
    (x+dx, y+dy),                                 # End of 2nd straight line    
    (x+0.9*idlerstepx+dx/2, y-0.9*idlerstepy+dy/2),       # Control point curve2
    (x, y),                                       # End of curve2 - FINISH
    ]
    codes = [Path.MOVETO,
         Path.LINETO,     # Start position
         Path.CURVE3,     # Control point for curve1
         Path.CURVE3,     # End of curve1
         Path.LINETO,
         Path.CURVE3,     # Control point for curve2
         Path.CURVE3,     # End of curve2       # Finish       
     ]
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(path, edgecolor='none', facecolor='black')
    plt.gca().add_patch(patch) 

# Draw a Straight Arrow
# x,y,dx,dy,etc
#plt.gca().add_patch(mpatches.Arrow(lm+3.88,y+0.4,0,-0.4,width=0.1,color='black', zorder=9))    # x,y,dx,dy,width
    
# Draw a Curved Arrow
    cs="arc3, rad="+str(r)
    curvarrow = mpatches.FancyArrowPatch(
        (x1, y1),
        (x2, y2),
        connectionstyle=cs,    # Default
        mutation_scale=2*abs(mag)*pltwid,
        facecolor='blue',
        edgecolor=arrowcolor,
        )
    plt.gca().add_patch(curvarrow)
    
# Path generating code for star
def Draw_Star(xc,yc,scale):
    # It is hideously complicated to create a 'cleaned' list of verts and codes
    # so I generate it with a program and insert the numbers
    
    oradius = scale
    iradius = 0.75 * scale
    spikes = 27
   
    #Draw a path with straight lines and curves
    vs = 'verts = [\n'
    cs = 'codes = [\n'
    for z in range (0,spikes+1):
        # Add outer point to path
        xe, ye = Rotate (xc, yc, xc, yc+iradius, z*2*pi/spikes)
        vs += '(' + str(truncate(xe,3)) + ', ' + str(truncate(ye,3) + '),\n')        
        if z == 0:
            cs += 'Path.MOVETO,\n'        
        else:
            cs += 'Path.LINETO,\n'
        # Add inner point to path
        xe, ye = Rotate (xc, yc, xc, yc+oradius, (z+0.5)*2*pi/spikes)
        vs += '(' + str(truncate(xe,3)) + ', ' + str(truncate(ye,3) + '),\n')             
        cs += 'Path.LINETO,\n'
        
    vs += ']'
    cs += ']'
    print (vs)
    print (cs)
        
    # Paste in list of verts and codes here 

     
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(path, edgecolor='orange', facecolor='yellow', linewidth=3)
    plt.gca().add_patch(patch) 
     
    
"""